param (
    [String] $Secret = $null,
    [String] $WebState = $null,
    [Switch] $help = $false,
    [Switch] $ActiveMQ = $false,
    [Switch] $All = $false,
    [Switch] $Base = $false,
    [Switch] $Everything = $false,
    [Switch] $Gov = $false,
    [Switch] $Governance = $false,
    [Switch] $IDGov = $false,
    [Switch] $Java = $false,
    [Switch] $JDK = $false,
    [Switch] $JRE = $false,
    [Switch] $Mail = $false,
    [Switch] $PG = $false,
    [Switch] $Postgres = $false,
    [Switch] $PostgreSQL = $false,
    [Switch] $Tomcat = $false,
    [Switch] $Web = $false
)

Add-Type -AssemblyName System.IO.Compression.FileSystem

#Versions
$VSN_ACTIVEMQ           = "5.16.5"
$VSN_JDK                = "8.66.0.15-ca-jdk8.0.352"
$VSN_POSTGRESQL         = "11.18-1"
$VSN_POSTGRESQL_SERVICE = "11"
$VSN_TOMCAT             = "9.0.69"

#Names
$NAME_ACTIVEMQ          = "activemq"
$NAME_POSTGRES          = "postgres"
$NAME_TOMCAT            = "tomcat"
$NAME_TOMCAT_LAUNCH     = "start-tomcat.bat"
$NAME_TOMCAT_QUIT       = "stop-tomcat.bat"
$NAME_TOMCAT_SERVICE    = "TomcatService.exe"

#Directories
$DIR_ACTIVEMQ           = "${NAME_ACTIVEMQ}"
$DIR_BACKUP             = "backup"
$DIR_BASE               = "c:\netiq\idm\apps"
$DIR_COMPRESSED         = "${PSScriptRoot}\compressed"
$DIR_JDK                = "jdk"
$DIR_LOG_BASE           = "${PSScriptRoot}"
$DIR_LOG_PARENT         = "logs"
$DIR_LOG                = "${DIR_LOG_BASE}\${DIR_LOG_PARENT}"
$DIR_POSTGRES           = "${NAME_POSTGRES}"
$DIR_POSTGRES_DATA      = "data"
$DIR_PRISTINE           = "${PSScriptRoot}\untouched"
$DIR_TOMCAT             = "${NAME_TOMCAT}"

#Files
$FILE_ACTIVEMQ          = "${DIR_COMPRESSED}\apache-activemq-${VSN_ACTIVEMQ}-bin.zip"
$FILE_JDK               = "${DIR_COMPRESSED}\zulu${VSN_JDK}-win_x64.zip"
$FILE_POSTGRESQL        = "${DIR_COMPRESSED}\postgresql-${VSN_POSTGRESQL}-windows-x64.exe"
$FILE_LOG               = "${DIR_LOG}\IDGov-Core-Helper.log"
$FILE_TOMCAT            = "${DIR_COMPRESSED}\apache-tomcat-${VSN_TOMCAT}-windows-x64.zip"

#Services (final state can be `start`, `stop`, and `restart`)
$SERV_TOMCAT_SERVICE    = "IDM Apps Tomcat Service"
$SERV_TOMCAT_STATE      = ""

#Parameters
$PARAM_OSP_LDAP_TYPE    = "edir"
$PARAM_OSP_LOGGING      = "WARN"
$PARAM_POSTGRES_PORT    = "5432"
$PARAM_TOMCAT_SERVER    = "localhost"
$PARAM_TOMCAT_OPTS_COMMON_ALL = "-Dnetiqtomcatcustomsrvidentify=netiqtomcatcustomsrvidentify;-Djava.awt.headless=true;-Dcom.netiq.ism.config=${DIR_BASE}\${DIR_TOMCAT}\conf\ism-configuration.properties;"
$PARAM_TOMCAT_OPTS_OSP  = "-Dcom.netiq.osp.ext-context-file=${DIR_BASE}\osp\lib\osp-conf-${PARAM_OSP_LDAP_TYPE}.jar;-Dcom.netiq.idm.osp.logging.level=${PARAM_OSP_LOGGING};-Dcom.netiq.idm.osp.client.host=${PARAM_TOMCAT_SERVER};-Dcom.netiq.idm.osp.logging.file.dir=${DIR_BASE}\${DIR_TOMCAT}\logs;-Dcom.netiq.idm.osp.audit.enabled=false;"
$PARAM_TOMCAT_OPTS_COMMON_IDENT = "-Dlog4j.configuration=file:${DIR_BASE}\${DIR_TOMCAT}\conf\log4j.properties;-Dinternal.atlaslite.jcce.xml.w3c.XMLUtil.suppressSecurityWarning=true;"
$PARAM_TOMCAT_OPTS_GOV  = "-Dcom.netiq.ism.config.is.jndi.env=true;-Dnovell.logging.config.dir=${DIR_BASE}\${DIR_TOMCAT}\conf;"
$PARAM_TOMCAT_OPTS_RPT  = "-Dcom.netiq.rpt.config.file=${DIR_BASE}\${DIR_TOMCAT}\conf\ism-configuration.properties;-Dextend.local.config.dir=${DIR_BASE}\${DIR_TOMCAT}\conf;-Didmuserapp.logging.config.dir=${DIR_BASE}\${DIR_TOMCAT}\conf;"
$PARAM_TOMCAT_JVM_OPTS  = "${PARAM_TOMCAT_OPTS_COMMON_ALL}${PARAM_TOMCAT_OPTS_OSP}${PARAM_TOMCAT_OPTS_COMMON_IDENT}${PARAM_TOMCAT_OPTS_GOV}${PARAM_TOMCAT_OPTS_RPT}"
$PARAM_TOMCAT_JVM_MEM_INIT  = "1024"
$PARAM_TOMCAT_JVM_MEM_FINAL = "1024"

#Parse and consume input arguments provided
$USAGE_THIS = "Usage: " + $MyInvocation.MyCommand.Name + " -all|-gov|-base|-tomcat|-activeMQ|-postgreSQL|-java [<PG password>]"

$PARAM_INSTALL_ACTIVE_MQ = $false
$PARAM_INSTALL_JAVA = $false
$PARAM_INSTALL_POSTGRESQL = $false
$PARAM_INSTALL_WEB = $false

If ($help) {
    Write-Host "$USAGE_THIS"
    Exit
}
If ($All -or $Everything) {
    $PARAM_INSTALL_ACTIVE_MQ = $true
    $PARAM_INSTALL_JAVA = $true
    $PARAM_INSTALL_POSTGRESQL = $true
    $PARAM_INSTALL_WEB = $true
}
If ($Governance -or $Gov -or $IDGov) {
    $PARAM_INSTALL_ACTIVE_MQ = $true
    $PARAM_INSTALL_JAVA = $true
    $PARAM_INSTALL_WEB = $true
}
If ($Base -or $Tomcat -or $Web) {
    $PARAM_INSTALL_JAVA = $true
    $PARAM_INSTALL_WEB = $true
}
If ($ActiveMQ -or $Mail) {
    $PARAM_INSTALL_ACTIVE_MQ = $true
    $PARAM_INSTALL_JAVA = $true
}
If ($Java -or $JDK -or $JRE) {
    $PARAM_INSTALL_JAVA = $true
}
If ($Postgres -or $PostgreSQL -or $PG) {
    $PARAM_INSTALL_POSTGRESQL = $true
}

If (-not $WebState.equals("")) {
    $SERV_TOMCAT_STATE = $WebState
}

If (-not $Secret.equals("")) {
    $PARAM_POSTGRES_SECRET = $Secret
}

If (-not $PARAM_INSTALL_ACTIVE_MQ -and -not $PARAM_INSTALL_JAVA -and -not $PARAM_INSTALL_POSTGRESQL -and -not $PARAM_INSTALL_WEB) {
    Write-Host "Nothing specified for installation!"
    Write-Host "$USAGE_THIS"
    Exit
}
If ($PARAM_INSTALL_POSTGRESQL -and $PARAM_POSTGRES_SECRET -eq $null) {
    Write-Host "Required PostgreSQL password missing!"
    Write-Host "$USAGE_THIS"
    Exit
}

Function Unzip([string]${zip}, [string]${destination})
{
    ${private:parent} = Split-Path -Path ${destination} -Parent
    ${private:leaf} = Split-Path -Path ${destination} -Leaf
    If (-not (Test-Path ${destination})) {
        ${null} = (New-Item -Path "${parent}" -Name "${leaf}" -ItemType "directory")
    }
    Write-Host ""
    Write-Host "######################################################################################"
    Write-Host " Extracting `"${zip}`" into `"${destination}`":"
    Write-Host "######################################################################################"
    [System.IO.Compression.ZipFile]::ExtractToDirectory(${zip}, ${destination})
    ${private:children} = @(Get-ChildItem -Path "${destination}")
    If (($(${children}.count) -eq 1) -and (${children}[0].Attributes -match "Directory")) {
        # When the only child-item is a directory we should strip off a layer, which should be safe
        #   Otherwise the (extracted) files in this folder are those that should stay
        # On some systems we cannot rename the extracted folder immediately
        ${private:rename_waiting} = ${TRUE}
        ${private:rename_count} = 0
        While (${private:rename_waiting}) {
            try {
                Rename-Item -Path "$(${children}[0].FullName)" -newName "${leaf}" -ErrorAction Stop
                ${private:rename_waiting} = ${FALSE}
            } catch [System.IO.IOException] {
                If (${private:rename_count}++ -ge 15) {
                    # We have waited long enough; report the error
                    ${private:rename_waiting} = ${FALSE}
                    throw $_
                } Else {
                    Start-Sleep -Seconds 2
                }
            }
        }
        Get-ChildItem -Path "${destination}\${leaf}" -Recurse | Move-Item -Destination "${destination}"
        Remove-Item -Path "${destination}\${leaf}"
    }
}

Function Disable-TomcatAJPConnector {
  Write-Host ""
  Write-Host "######################################################################################"
  Write-Host " Disabling Tomcat AJP connector within `"${DIR_BASE}\${DIR_TOMCAT}\conf\server.xml`""
  Write-Host "######################################################################################"

  Set-Content `
    -Path "${DIR_BASE}\${DIR_TOMCAT}\conf\server.xml" `
    -Value "$((Get-Content `"${DIR_BASE}\${DIR_TOMCAT}\conf\server.xml`" -Raw) -replace -join `
      (
        "(?ms)",
        "(\s+<!-- Define an AJP 1\.3 Connector on port [0-9]+ -->)(\s*)", # $1 $2
        "(<Connector port=``"[0-9]+``" protocol=``"AJP/1\.3``" redirectPort=``"[0-9]+``"\s*/>)(\s*)" # $3 $4
      ), -join `
      (
        "`$1`$2",
        "<!--`$2",
        "`$3`$2",
        "-->",
        "`$2`$2"
      )
    )"
}

Function Enable-CatalinaOutput {
    Write-Host ""
    Write-Host "######################################################################################"
    Write-Host " Enable Output to CATALINA_OUT:"
    Write-Host "######################################################################################"
    $text = (Get-Content -Path "${DIR_BASE}\${DIR_TOMCAT}\bin\catalina.bat" -ReadCount 0) -join "`r`n"
    $text -replace 'set _EXECJAVA=start "%TITLE%" %_RUNJAVA%', 'set _EXECJAVA=cmd /c "%_RUNJAVA%' | Set-Content -Path "${DIR_BASE}\${DIR_TOMCAT}\bin\catalina.bat"
    $text = (Get-Content -Path "${DIR_BASE}\${DIR_TOMCAT}\bin\catalina.bat" -ReadCount 0) -join "`r`n"
    $text -replace '%ACTION%', '%ACTION%" >> %CATALINA_OUT% 2>&1' | Set-Content -Path "${DIR_BASE}\${DIR_TOMCAT}\bin\catalina.bat"
}

Function Install-ActiveMQ {
    Unzip "${FILE_ACTIVEMQ}" "${DIR_BASE}\${DIR_ACTIVEMQ}"

    ((Get-Content -path "${DIR_BASE}\${DIR_ACTIVEMQ}\bin\activemq.bat" -Raw) -replace
        "if `"%JAVA_HOME%`" == `"`" goto noJavaHome",
        "if `"%JAVA_HOME%`" == `"`" set JAVA_HOME=${DIR_BASE}\jre") |
        Set-Content -Path "${DIR_BASE}\${DIR_ACTIVEMQ}\bin\activemq.bat"
}

Function Install-JDK {
    Unzip "${FILE_JDK}" "${DIR_BASE}\${DIR_JDK}"

    Write-Host ""
    Write-Host "######################################################################################"
    Write-Host " Creating symbolic link at ${DIR_BASE}\jre for ${DIR_JDK}\jre"
    Write-Host "######################################################################################"
    $null = (New-Item -ItemType SymbolicLink -Path "${DIR_BASE}\jre" -Target "${DIR_BASE}\${DIR_JDK}\jre")
}

Function Install-PostgreSQL {
    Write-Host ""
    Write-Host "######################################################################################"
    Write-Host " Installing PostgreSQL to ${DIR_BASE}\${DIR_POSTGRES}:"
    Write-Host "######################################################################################"
    Start-Process ${FILE_POSTGRESQL} -Wait -NoNewWindow -ArgumentList `
        "--unattendedmodeui none", `
        "--mode unattended", `
        "--superaccount ${NAME_POSTGRES}", `
        "--superpassword ${PARAM_POSTGRES_SECRET}", `
        "--serverport ${PARAM_POSTGRES_PORT}", `
        "--prefix ${DIR_BASE}\${DIR_POSTGRES}", `
        "--datadir ${DIR_BASE}\${DIR_POSTGRES}\${DIR_POSTGRES_DATA}"

    Write-Host ""
    Write-Host "######################################################################################"
    Write-Host " Updating PostgreSQL to allow remote access"
    Write-Host "######################################################################################"
    $text = (Get-Content -Path "${DIR_BASE}\${DIR_POSTGRES}\${DIR_POSTGRES_DATA}\pg_hba.conf" -ReadCount 0) -join "`n"
    $text -replace '/32', '/0 ' | Set-Content -Path "${DIR_BASE}\${DIR_POSTGRES}\${DIR_POSTGRES_DATA}\pg_hba.conf"

    Write-Host ""
    Write-Host "######################################################################################"
    Write-Host " Restarting PostgreSQL"
    Write-Host "######################################################################################"
    Restart-Service -Name postgresql-x64-${VSN_POSTGRESQL_SERVICE}
}

Function Install-Tomcat {
    Unzip "$FILE_TOMCAT" "${DIR_BASE}\${DIR_TOMCAT}"

    Write-Host ""
    Write-Host "######################################################################################"
    Write-Host " Moving the default Tomcat wars to ${DIR_BASE}\${DIR_BACKUP}:"
    Write-Host "######################################################################################"
    $null = (New-Item -Path "${DIR_BASE}" -Name "${DIR_BACKUP}" -ItemType "directory")
    Get-ChildItem -Path "${DIR_BASE}\${DIR_TOMCAT}\webapps" -Recurse | Move-Item -Destination "${DIR_BASE}\${DIR_BACKUP}"

    Write-Host ""
    Write-Host "######################################################################################"
    Write-Host " Creating new default Tomcat index file at ${DIR_BASE}\${DIR_TOMCAT}\webapps\ROOT\index.html:"
    Write-Host "######################################################################################"
    $null = (New-Item -Path "${DIR_BASE}\${DIR_TOMCAT}\webapps" -Name "ROOT" -ItemType "directory")
    Copy-Item -Path "${DIR_PRISTINE}\index.html" -Destination "${DIR_BASE}\${DIR_TOMCAT}\webapps\ROOT\index.html"

    Disable-TomcatAJPConnector
}

Function Join-ActiveMQTomcat {
    Write-Host ""
    Write-Host "######################################################################################"
    Write-Host " Copying the ActiveMQ jar to ${DIR_BASE}\${DIR_TOMCAT}\lib:"
    Write-Host "######################################################################################"
    Copy-Item "${DIR_BASE}\${DIR_ACTIVEMQ}\${DIR_ACTIVEMQ}-all*.jar" -Destination "${DIR_BASE}\${DIR_TOMCAT}\lib"
}

Function Setup-TomcatService {
    Write-Host ""
    Write-Host "######################################################################################"
    Write-Host " Prepare ${DIR_BASE}\${DIR_TOMCAT}\bin\${NAME_TOMCAT_LAUNCH}:"
    Write-Host "######################################################################################"
    Copy-Item -Path "${DIR_PRISTINE}\${NAME_TOMCAT_LAUNCH}" -Destination "${DIR_BASE}\${DIR_TOMCAT}\bin\${NAME_TOMCAT_LAUNCH}"
    $text = (Get-Content -Path "${DIR_BASE}\${DIR_TOMCAT}\bin\${NAME_TOMCAT_LAUNCH}" -ReadCount 0) -join "`r`n"
    $text -replace "REPLACE_INSTALL_BASE", "${DIR_BASE}" | Set-Content -Path "${DIR_BASE}\${DIR_TOMCAT}\bin\${NAME_TOMCAT_LAUNCH}"
    $text = (Get-Content -Path "${DIR_BASE}\${DIR_TOMCAT}\bin\${NAME_TOMCAT_LAUNCH}" -ReadCount 0) -join "`r`n"
    $text -replace "REPLACE_INSTALL_JAVA", "${DIR_BASE}\${DIR_JDK}" | Set-Content -Path "${DIR_BASE}\${DIR_TOMCAT}\bin\${NAME_TOMCAT_LAUNCH}"

    Write-Host ""
    Write-Host "######################################################################################"
    Write-Host " Prepare ${DIR_BASE}\${DIR_TOMCAT}\bin\${NAME_TOMCAT_QUIT}:"
    Write-Host "######################################################################################"
    Copy-Item -Path "${DIR_PRISTINE}\${NAME_TOMCAT_QUIT}" -Destination "${DIR_BASE}\${DIR_TOMCAT}\bin\${NAME_TOMCAT_QUIT}"
    $text = (Get-Content -Path "${DIR_BASE}\${DIR_TOMCAT}\bin\${NAME_TOMCAT_QUIT}" -ReadCount 0) -join "`r`n"
    $text -replace "REPLACE_INSTALL_BASE", "${DIR_BASE}" | Set-Content -Path "${DIR_BASE}\${DIR_TOMCAT}\bin\${NAME_TOMCAT_QUIT}"
    $text = (Get-Content -Path "${DIR_BASE}\${DIR_TOMCAT}\bin\${NAME_TOMCAT_QUIT}" -ReadCount 0) -join "`r`n"
    $text -replace "REPLACE_INSTALL_JAVA", "${DIR_BASE}\${DIR_JDK}" | Set-Content -Path "${DIR_BASE}\${DIR_TOMCAT}\bin\${NAME_TOMCAT_QUIT}"

    Write-Host ""
    Write-Host "######################################################################################"
    Write-Host " Create the Tomcat service ${SERV_TOMCAT_SERVICE}:"
    Write-Host "######################################################################################"
    Copy-Item -Path "${DIR_PRISTINE}\${NAME_TOMCAT_SERVICE}" -Destination "${DIR_BASE}\${DIR_TOMCAT}\bin\${NAME_TOMCAT_SERVICE}"
    New-Service -Name "${SERV_TOMCAT_SERVICE}" -BinaryPathName "${DIR_BASE}\${DIR_TOMCAT}\bin\${NAME_TOMCAT_SERVICE} ${DIR_BASE}" -StartupType Manual
}

Function Restart-TomcatService {
    Write-Host ""
    Write-Host "######################################################################################"
    Write-Host " Restart the Tomcat service, ${SERV_TOMCAT_SERVICE}:"
    Write-Host "######################################################################################"
    Restart-Service -Name "${SERV_TOMCAT_SERVICE}"
}

Function Start-TomcatService {
    Write-Host ""
    Write-Host "######################################################################################"
    Write-Host " Start the Tomcat service, ${SERV_TOMCAT_SERVICE}:"
    Write-Host "######################################################################################"
    Start-Service -Name "${SERV_TOMCAT_SERVICE}"
}

Function Stop-TomcatService {
    Write-Host ""
    Write-Host "######################################################################################"
    Write-Host " Stop the Tomcat service, ${SERV_TOMCAT_SERVICE}:"
    Write-Host "######################################################################################"
    Stop-Service -Name "${SERV_TOMCAT_SERVICE}"
}

Function Start-Recording {
    Write-Host ""
    Write-Host "######################################################################################"
    Write-Host " Stopping existing transcripts"
    Write-Host "######################################################################################"
    $ErrorActionPreference="SilentlyContinue"
    Stop-Transcript | out-null

    Write-Host ""
    Write-Host "######################################################################################"
    Write-Host " Creating log directory at: ${DIR_LOG}"
    Write-Host "######################################################################################"
    $null = (New-Item -Path "${DIR_LOG_BASE}" -Name "${DIR_LOG_PARENT}" -ItemType "directory")

    Write-Host ""
    Write-Host "######################################################################################"
    Write-Host " Removing previous log if exists: ${FILE_LOG}"
    Write-Host "######################################################################################"
    Remove-Item -Path "${FILE_LOG}"

    Write-Host ""
    Write-Host "######################################################################################"
    Write-Host " Starting transcript: ${FILE_LOG}"
    Write-Host "######################################################################################"
    $ErrorActionPreference = "Continue"
    Start-Transcript -path "${FILE_LOG}" -append
}

Function Stop-Recording {
    Write-Host ""
    Write-Host "######################################################################################"
    Write-Host " Complete the transcript"
    Write-Host "######################################################################################"
    Stop-Transcript

    Write-Host ""
    Write-Host "######################################################################################"
    Write-Host " Completed"
    Write-Host " Review the log file: ${FILE_LOG}"
    Write-Host "######################################################################################"
}

Start-Recording
If ($PARAM_INSTALL_JAVA) { Install-JDK }
If ($PARAM_INSTALL_ACTIVE_MQ) { Install-ActiveMQ }
If ($PARAM_INSTALL_WEB) {
    Install-Tomcat
    Enable-CatalinaOutput
    If ($PARAM_INSTALL_ACTIVE_MQ) { Join-ActiveMQTomcat }
    Setup-TomcatService
    If ($SERV_TOMCAT_STATE.equals("start")) { Start-TomcatService }
    ElseIf ($SERV_TOMCAT_STATE.equals("restart")) { Restart-TomcatService }
    ElseIf ($SERV_TOMCAT_STATE.equals("stop")) { Stop-TomcatService }
}
if ($PARAM_INSTALL_POSTGRESQL) { Install-PostgreSQL }
Stop-Recording


# SIG # Begin signature block
# MIIsugYJKoZIhvcNAQcCoIIsqzCCLKcCAQExDzANBglghkgBZQMEAgEFADB5Bgor
# BgEEAYI3AgEEoGswaTA0BgorBgEEAYI3AgEeMCYCAwEAAAQQH8w7YFlLCE63JNLG
# KX7zUQIBAAIBAAIBAAIBAAIBADAxMA0GCWCGSAFlAwQCAQUABCAHtc/rUADS7vid
# d4HcH7EtXT2eDdrCYvAvGk9Vo/O2W6CCJj0wggWNMIIEdaADAgECAhAOmxiO+dAt
# 5+/bUOIIQBhaMA0GCSqGSIb3DQEBDAUAMGUxCzAJBgNVBAYTAlVTMRUwEwYDVQQK
# EwxEaWdpQ2VydCBJbmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5jb20xJDAiBgNV
# BAMTG0RpZ2lDZXJ0IEFzc3VyZWQgSUQgUm9vdCBDQTAeFw0yMjA4MDEwMDAwMDBa
# Fw0zMTExMDkyMzU5NTlaMGIxCzAJBgNVBAYTAlVTMRUwEwYDVQQKEwxEaWdpQ2Vy
# dCBJbmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5jb20xITAfBgNVBAMTGERpZ2lD
# ZXJ0IFRydXN0ZWQgUm9vdCBHNDCCAiIwDQYJKoZIhvcNAQEBBQADggIPADCCAgoC
# ggIBAL/mkHNo3rvkXUo8MCIwaTPswqclLskhPfKK2FnC4SmnPVirdprNrnsbhA3E
# MB/zG6Q4FutWxpdtHauyefLKEdLkX9YFPFIPUh/GnhWlfr6fqVcWWVVyr2iTcMKy
# unWZanMylNEQRBAu34LzB4TmdDttceItDBvuINXJIB1jKS3O7F5OyJP4IWGbNOsF
# xl7sWxq868nPzaw0QF+xembud8hIqGZXV59UWI4MK7dPpzDZVu7Ke13jrclPXuU1
# 5zHL2pNe3I6PgNq2kZhAkHnDeMe2scS1ahg4AxCN2NQ3pC4FfYj1gj4QkXCrVYJB
# MtfbBHMqbpEBfCFM1LyuGwN1XXhm2ToxRJozQL8I11pJpMLmqaBn3aQnvKFPObUR
# WBf3JFxGj2T3wWmIdph2PVldQnaHiZdpekjw4KISG2aadMreSx7nDmOu5tTvkpI6
# nj3cAORFJYm2mkQZK37AlLTSYW3rM9nF30sEAMx9HJXDj/chsrIRt7t/8tWMcCxB
# YKqxYxhElRp2Yn72gLD76GSmM9GJB+G9t+ZDpBi4pncB4Q+UDCEdslQpJYls5Q5S
# UUd0viastkF13nqsX40/ybzTQRESW+UQUOsxxcpyFiIJ33xMdT9j7CFfxCBRa2+x
# q4aLT8LWRV+dIPyhHsXAj6KxfgommfXkaS+YHS312amyHeUbAgMBAAGjggE6MIIB
# NjAPBgNVHRMBAf8EBTADAQH/MB0GA1UdDgQWBBTs1+OC0nFdZEzfLmc/57qYrhwP
# TzAfBgNVHSMEGDAWgBRF66Kv9JLLgjEtUYunpyGd823IDzAOBgNVHQ8BAf8EBAMC
# AYYweQYIKwYBBQUHAQEEbTBrMCQGCCsGAQUFBzABhhhodHRwOi8vb2NzcC5kaWdp
# Y2VydC5jb20wQwYIKwYBBQUHMAKGN2h0dHA6Ly9jYWNlcnRzLmRpZ2ljZXJ0LmNv
# bS9EaWdpQ2VydEFzc3VyZWRJRFJvb3RDQS5jcnQwRQYDVR0fBD4wPDA6oDigNoY0
# aHR0cDovL2NybDMuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0QXNzdXJlZElEUm9vdENB
# LmNybDARBgNVHSAECjAIMAYGBFUdIAAwDQYJKoZIhvcNAQEMBQADggEBAHCgv0Nc
# Vec4X6CjdBs9thbX979XB72arKGHLOyFXqkauyL4hxppVCLtpIh3bb0aFPQTSnov
# Lbc47/T/gLn4offyct4kvFIDyE7QKt76LVbP+fT3rDB6mouyXtTP0UNEm0Mh65Zy
# oUi0mcudT6cGAxN3J0TU53/oWajwvy8LpunyNDzs9wPHh6jSTEAZNUZqaVSwuKFW
# juyk1T3osdz9HNj0d1pcVIxv76FQPfx2CWiEn2/K2yCNNWAcAgPLILCsWKAOQGPF
# mCLBsln1VWvPJ6tsds5vIy30fnFqI2si/xK4VC0nftg62fC2h5b9W9FcrBjDTZ9z
# twGpn1eqXijiuZQwggWQMIIDeKADAgECAhAFmxtXno4hMuI5B72nd3VcMA0GCSqG
# SIb3DQEBDAUAMGIxCzAJBgNVBAYTAlVTMRUwEwYDVQQKEwxEaWdpQ2VydCBJbmMx
# GTAXBgNVBAsTEHd3dy5kaWdpY2VydC5jb20xITAfBgNVBAMTGERpZ2lDZXJ0IFRy
# dXN0ZWQgUm9vdCBHNDAeFw0xMzA4MDExMjAwMDBaFw0zODAxMTUxMjAwMDBaMGIx
# CzAJBgNVBAYTAlVTMRUwEwYDVQQKEwxEaWdpQ2VydCBJbmMxGTAXBgNVBAsTEHd3
# dy5kaWdpY2VydC5jb20xITAfBgNVBAMTGERpZ2lDZXJ0IFRydXN0ZWQgUm9vdCBH
# NDCCAiIwDQYJKoZIhvcNAQEBBQADggIPADCCAgoCggIBAL/mkHNo3rvkXUo8MCIw
# aTPswqclLskhPfKK2FnC4SmnPVirdprNrnsbhA3EMB/zG6Q4FutWxpdtHauyefLK
# EdLkX9YFPFIPUh/GnhWlfr6fqVcWWVVyr2iTcMKyunWZanMylNEQRBAu34LzB4Tm
# dDttceItDBvuINXJIB1jKS3O7F5OyJP4IWGbNOsFxl7sWxq868nPzaw0QF+xembu
# d8hIqGZXV59UWI4MK7dPpzDZVu7Ke13jrclPXuU15zHL2pNe3I6PgNq2kZhAkHnD
# eMe2scS1ahg4AxCN2NQ3pC4FfYj1gj4QkXCrVYJBMtfbBHMqbpEBfCFM1LyuGwN1
# XXhm2ToxRJozQL8I11pJpMLmqaBn3aQnvKFPObURWBf3JFxGj2T3wWmIdph2PVld
# QnaHiZdpekjw4KISG2aadMreSx7nDmOu5tTvkpI6nj3cAORFJYm2mkQZK37AlLTS
# YW3rM9nF30sEAMx9HJXDj/chsrIRt7t/8tWMcCxBYKqxYxhElRp2Yn72gLD76GSm
# M9GJB+G9t+ZDpBi4pncB4Q+UDCEdslQpJYls5Q5SUUd0viastkF13nqsX40/ybzT
# QRESW+UQUOsxxcpyFiIJ33xMdT9j7CFfxCBRa2+xq4aLT8LWRV+dIPyhHsXAj6Kx
# fgommfXkaS+YHS312amyHeUbAgMBAAGjQjBAMA8GA1UdEwEB/wQFMAMBAf8wDgYD
# VR0PAQH/BAQDAgGGMB0GA1UdDgQWBBTs1+OC0nFdZEzfLmc/57qYrhwPTzANBgkq
# hkiG9w0BAQwFAAOCAgEAu2HZfalsvhfEkRvDoaIAjeNkaA9Wz3eucPn9mkqZucl4
# XAwMX+TmFClWCzZJXURj4K2clhhmGyMNPXnpbWvWVPjSPMFDQK4dUPVS/JA7u5iZ
# aWvHwaeoaKQn3J35J64whbn2Z006Po9ZOSJTROvIXQPK7VB6fWIhCoDIc2bRoAVg
# X+iltKevqPdtNZx8WorWojiZ83iL9E3SIAveBO6Mm0eBcg3AFDLvMFkuruBx8lbk
# apdvklBtlo1oepqyNhR6BvIkuQkRUNcIsbiJeoQjYUIp5aPNoiBB19GcZNnqJqGL
# FNdMGbJQQXE9P01wI4YMStyB0swylIQNCAmXHE/A7msgdDDS4Dk0EIUhFQEI6FUy
# 3nFJ2SgXUE3mvk3RdazQyvtBuEOlqtPDBURPLDab4vriRbgjU2wGb2dVf0a1TD9u
# KFp5JtKkqGKX0h7i7UqLvBv9R0oN32dmfrJbQdA75PQ79ARj6e/CVABRoIoqyc54
# zNXqhwQYs86vSYiv85KZtrPmYQ/ShQDnUBrkG5WdGaG5nLGbsQAe79APT0JsyQq8
# 7kP6OnGlyE0mpTX9iV28hWIdMtKgK1TtmlfB2/oQzxm3i0objwG2J5VT6LaJbVu8
# aNQj6ItRolb58KaAoNYes7wPD1N1KarqE3fk3oyBIa0HEEcRrYc9B9F1vM/zZn4w
# ggauMIIElqADAgECAhAHNje3JFR82Ees/ShmKl5bMA0GCSqGSIb3DQEBCwUAMGIx
# CzAJBgNVBAYTAlVTMRUwEwYDVQQKEwxEaWdpQ2VydCBJbmMxGTAXBgNVBAsTEHd3
# dy5kaWdpY2VydC5jb20xITAfBgNVBAMTGERpZ2lDZXJ0IFRydXN0ZWQgUm9vdCBH
# NDAeFw0yMjAzMjMwMDAwMDBaFw0zNzAzMjIyMzU5NTlaMGMxCzAJBgNVBAYTAlVT
# MRcwFQYDVQQKEw5EaWdpQ2VydCwgSW5jLjE7MDkGA1UEAxMyRGlnaUNlcnQgVHJ1
# c3RlZCBHNCBSU0E0MDk2IFNIQTI1NiBUaW1lU3RhbXBpbmcgQ0EwggIiMA0GCSqG
# SIb3DQEBAQUAA4ICDwAwggIKAoICAQDGhjUGSbPBPXJJUVXHJQPE8pE3qZdRodbS
# g9GeTKJtoLDMg/la9hGhRBVCX6SI82j6ffOciQt/nR+eDzMfUBMLJnOWbfhXqAJ9
# /UO0hNoR8XOxs+4rgISKIhjf69o9xBd/qxkrPkLcZ47qUT3w1lbU5ygt69OxtXXn
# HwZljZQp09nsad/ZkIdGAHvbREGJ3HxqV3rwN3mfXazL6IRktFLydkf3YYMZ3V+0
# VAshaG43IbtArF+y3kp9zvU5EmfvDqVjbOSmxR3NNg1c1eYbqMFkdECnwHLFuk4f
# sbVYTXn+149zk6wsOeKlSNbwsDETqVcplicu9Yemj052FVUmcJgmf6AaRyBD40Nj
# gHt1biclkJg6OBGz9vae5jtb7IHeIhTZgirHkr+g3uM+onP65x9abJTyUpURK1h0
# QCirc0PO30qhHGs4xSnzyqqWc0Jon7ZGs506o9UD4L/wojzKQtwYSH8UNM/STKvv
# mz3+DrhkKvp1KCRB7UK/BZxmSVJQ9FHzNklNiyDSLFc1eSuo80VgvCONWPfcYd6T
# /jnA+bIwpUzX6ZhKWD7TA4j+s4/TXkt2ElGTyYwMO1uKIqjBJgj5FBASA31fI7tk
# 42PgpuE+9sJ0sj8eCXbsq11GdeJgo1gJASgADoRU7s7pXcheMBK9Rp6103a50g5r
# mQzSM7TNsQIDAQABo4IBXTCCAVkwEgYDVR0TAQH/BAgwBgEB/wIBADAdBgNVHQ4E
# FgQUuhbZbU2FL3MpdpovdYxqII+eyG8wHwYDVR0jBBgwFoAU7NfjgtJxXWRM3y5n
# P+e6mK4cD08wDgYDVR0PAQH/BAQDAgGGMBMGA1UdJQQMMAoGCCsGAQUFBwMIMHcG
# CCsGAQUFBwEBBGswaTAkBggrBgEFBQcwAYYYaHR0cDovL29jc3AuZGlnaWNlcnQu
# Y29tMEEGCCsGAQUFBzAChjVodHRwOi8vY2FjZXJ0cy5kaWdpY2VydC5jb20vRGln
# aUNlcnRUcnVzdGVkUm9vdEc0LmNydDBDBgNVHR8EPDA6MDigNqA0hjJodHRwOi8v
# Y3JsMy5kaWdpY2VydC5jb20vRGlnaUNlcnRUcnVzdGVkUm9vdEc0LmNybDAgBgNV
# HSAEGTAXMAgGBmeBDAEEAjALBglghkgBhv1sBwEwDQYJKoZIhvcNAQELBQADggIB
# AH1ZjsCTtm+YqUQiAX5m1tghQuGwGC4QTRPPMFPOvxj7x1Bd4ksp+3CKDaopafxp
# wc8dB+k+YMjYC+VcW9dth/qEICU0MWfNthKWb8RQTGIdDAiCqBa9qVbPFXONASIl
# zpVpP0d3+3J0FNf/q0+KLHqrhc1DX+1gtqpPkWaeLJ7giqzl/Yy8ZCaHbJK9nXzQ
# cAp876i8dU+6WvepELJd6f8oVInw1YpxdmXazPByoyP6wCeCRK6ZJxurJB4mwbfe
# Kuv2nrF5mYGjVoarCkXJ38SNoOeY+/umnXKvxMfBwWpx2cYTgAnEtp/Nh4cku0+j
# Sbl3ZpHxcpzpSwJSpzd+k1OsOx0ISQ+UzTl63f8lY5knLD0/a6fxZsNBzU+2QJsh
# IUDQtxMkzdwdeDrknq3lNHGS1yZr5Dhzq6YBT70/O3itTK37xJV77QpfMzmHQXh6
# OOmc4d0j/R0o08f56PGYX/sr2H7yRp11LB4nLCbbbxV7HhmLNriT1ObyF5lZynDw
# N7+YAN8gFk8n+2BnFqFmut1VwDophrCYoCvtlUG3OtUVmDG0YgkPCr2B2RP+v6TR
# 81fZvAT6gt4y3wSJ8ADNXcL50CN/AAvkdgIm2fBldkKmKYcJRyvmfxqkhQ/8mJb2
# VVQrH4D6wPIOK+XW+6kvRBVK5xMOHds3OBqhK/bt1nz8MIIGsDCCBJigAwIBAgIQ
# CK1AsmDSnEyfXs2pvZOu2TANBgkqhkiG9w0BAQwFADBiMQswCQYDVQQGEwJVUzEV
# MBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYDVQQLExB3d3cuZGlnaWNlcnQuY29t
# MSEwHwYDVQQDExhEaWdpQ2VydCBUcnVzdGVkIFJvb3QgRzQwHhcNMjEwNDI5MDAw
# MDAwWhcNMzYwNDI4MjM1OTU5WjBpMQswCQYDVQQGEwJVUzEXMBUGA1UEChMORGln
# aUNlcnQsIEluYy4xQTA/BgNVBAMTOERpZ2lDZXJ0IFRydXN0ZWQgRzQgQ29kZSBT
# aWduaW5nIFJTQTQwOTYgU0hBMzg0IDIwMjEgQ0ExMIICIjANBgkqhkiG9w0BAQEF
# AAOCAg8AMIICCgKCAgEA1bQvQtAorXi3XdU5WRuxiEL1M4zrPYGXcMW7xIUmMJ+k
# jmjYXPXrNCQH4UtP03hD9BfXHtr50tVnGlJPDqFX/IiZwZHMgQM+TXAkZLON4gh9
# NH1MgFcSa0OamfLFOx/y78tHWhOmTLMBICXzENOLsvsI8IrgnQnAZaf6mIBJNYc9
# URnokCF4RS6hnyzhGMIazMXuk0lwQjKP+8bqHPNlaJGiTUyCEUhSaN4QvRRXXegY
# E2XFf7JPhSxIpFaENdb5LpyqABXRN/4aBpTCfMjqGzLmysL0p6MDDnSlrzm2q2AS
# 4+jWufcx4dyt5Big2MEjR0ezoQ9uo6ttmAaDG7dqZy3SvUQakhCBj7A7CdfHmzJa
# wv9qYFSLScGT7eG0XOBv6yb5jNWy+TgQ5urOkfW+0/tvk2E0XLyTRSiDNipmKF+w
# c86LJiUGsoPUXPYVGUztYuBeM/Lo6OwKp7ADK5GyNnm+960IHnWmZcy740hQ83eR
# Gv7bUKJGyGFYmPV8AhY8gyitOYbs1LcNU9D4R+Z1MI3sMJN2FKZbS110YU0/EpF2
# 3r9Yy3IQKUHw1cVtJnZoEUETWJrcJisB9IlNWdt4z4FKPkBHX8mBUHOFECMhWWCK
# ZFTBzCEa6DgZfGYczXg4RTCZT/9jT0y7qg0IU0F8WD1Hs/q27IwyCQLMbDwMVhEC
# AwEAAaOCAVkwggFVMBIGA1UdEwEB/wQIMAYBAf8CAQAwHQYDVR0OBBYEFGg34Ou2
# O/hfEYb7/mF7CIhl9E5CMB8GA1UdIwQYMBaAFOzX44LScV1kTN8uZz/nupiuHA9P
# MA4GA1UdDwEB/wQEAwIBhjATBgNVHSUEDDAKBggrBgEFBQcDAzB3BggrBgEFBQcB
# AQRrMGkwJAYIKwYBBQUHMAGGGGh0dHA6Ly9vY3NwLmRpZ2ljZXJ0LmNvbTBBBggr
# BgEFBQcwAoY1aHR0cDovL2NhY2VydHMuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0VHJ1
# c3RlZFJvb3RHNC5jcnQwQwYDVR0fBDwwOjA4oDagNIYyaHR0cDovL2NybDMuZGln
# aWNlcnQuY29tL0RpZ2lDZXJ0VHJ1c3RlZFJvb3RHNC5jcmwwHAYDVR0gBBUwEzAH
# BgVngQwBAzAIBgZngQwBBAEwDQYJKoZIhvcNAQEMBQADggIBADojRD2NCHbuj7w6
# mdNW4AIapfhINPMstuZ0ZveUcrEAyq9sMCcTEp6QRJ9L/Z6jfCbVN7w6XUhtldU/
# SfQnuxaBRVD9nL22heB2fjdxyyL3WqqQz/WTauPrINHVUHmImoqKwba9oUgYftzY
# gBoRGRjNYZmBVvbJ43bnxOQbX0P4PpT/djk9ntSZz0rdKOtfJqGVWEjVGv7XJz/9
# kNF2ht0csGBc8w2o7uCJob054ThO2m67Np375SFTWsPK6Wrxoj7bQ7gzyE84FJKZ
# 9d3OVG3ZXQIUH0AzfAPilbLCIXVzUstG2MQ0HKKlS43Nb3Y3LIU/Gs4m6Ri+kAew
# Q3+ViCCCcPDMyu/9KTVcH4k4Vfc3iosJocsL6TEa/y4ZXDlx4b6cpwoG1iZnt5Lm
# Tl/eeqxJzy6kdJKt2zyknIYf48FWGysj/4+16oh7cGvmoLr9Oj9FpsToFpFSi0HA
# SIRLlk2rREDjjfAVKM7t8RhWByovEMQMCGQ8M4+uKIw8y4+ICw2/O/TOHnuO77Xr
# y7fwdxPm5yg/rBKupS8ibEH5glwVZsxsDsrFhsP2JjMMB0ug0wcCampAMEhLNKhR
# ILutG4UI4lkNbcoFUCvqShyepf2gpx8GdOfy1lKQ/a+FSCH5Vzu0nAPthkX0tGFu
# v2jiJmCG6sivqf6UHedjGzqGVnhOMIIGwDCCBKigAwIBAgIQDE1pckuU+jwqSj0p
# B4A9WjANBgkqhkiG9w0BAQsFADBjMQswCQYDVQQGEwJVUzEXMBUGA1UEChMORGln
# aUNlcnQsIEluYy4xOzA5BgNVBAMTMkRpZ2lDZXJ0IFRydXN0ZWQgRzQgUlNBNDA5
# NiBTSEEyNTYgVGltZVN0YW1waW5nIENBMB4XDTIyMDkyMTAwMDAwMFoXDTMzMTEy
# MTIzNTk1OVowRjELMAkGA1UEBhMCVVMxETAPBgNVBAoTCERpZ2lDZXJ0MSQwIgYD
# VQQDExtEaWdpQ2VydCBUaW1lc3RhbXAgMjAyMiAtIDIwggIiMA0GCSqGSIb3DQEB
# AQUAA4ICDwAwggIKAoICAQDP7KUmOsap8mu7jcENmtuh6BSFdDMaJqzQHFUeHjZt
# vJJVDGH0nQl3PRWWCC9rZKT9BoMW15GSOBwxApb7crGXOlWvM+xhiummKNuQY1y9
# iVPgOi2Mh0KuJqTku3h4uXoW4VbGwLpkU7sqFudQSLuIaQyIxvG+4C99O7HKU41A
# gx7ny3JJKB5MgB6FVueF7fJhvKo6B332q27lZt3iXPUv7Y3UTZWEaOOAy2p50dIQ
# kUYp6z4m8rSMzUy5Zsi7qlA4DeWMlF0ZWr/1e0BubxaompyVR4aFeT4MXmaMGgok
# vpyq0py2909ueMQoP6McD1AGN7oI2TWmtR7aeFgdOej4TJEQln5N4d3CraV++C0b
# H+wrRhijGfY59/XBT3EuiQMRoku7mL/6T+R7Nu8GRORV/zbq5Xwx5/PCUsTmFnta
# fqUlc9vAapkhLWPlWfVNL5AfJ7fSqxTlOGaHUQhr+1NDOdBk+lbP4PQK5hRtZHi7
# mP2Uw3Mh8y/CLiDXgazT8QfU4b3ZXUtuMZQpi+ZBpGWUwFjl5S4pkKa3YWT62SBs
# GFFguqaBDwklU/G/O+mrBw5qBzliGcnWhX8T2Y15z2LF7OF7ucxnEweawXjtxojI
# sG4yeccLWYONxu71LHx7jstkifGxxLjnU15fVdJ9GSlZA076XepFcxyEftfO4tQ6
# dwIDAQABo4IBizCCAYcwDgYDVR0PAQH/BAQDAgeAMAwGA1UdEwEB/wQCMAAwFgYD
# VR0lAQH/BAwwCgYIKwYBBQUHAwgwIAYDVR0gBBkwFzAIBgZngQwBBAIwCwYJYIZI
# AYb9bAcBMB8GA1UdIwQYMBaAFLoW2W1NhS9zKXaaL3WMaiCPnshvMB0GA1UdDgQW
# BBRiit7QYfyPMRTtlwvNPSqUFN9SnDBaBgNVHR8EUzBRME+gTaBLhklodHRwOi8v
# Y3JsMy5kaWdpY2VydC5jb20vRGlnaUNlcnRUcnVzdGVkRzRSU0E0MDk2U0hBMjU2
# VGltZVN0YW1waW5nQ0EuY3JsMIGQBggrBgEFBQcBAQSBgzCBgDAkBggrBgEFBQcw
# AYYYaHR0cDovL29jc3AuZGlnaWNlcnQuY29tMFgGCCsGAQUFBzAChkxodHRwOi8v
# Y2FjZXJ0cy5kaWdpY2VydC5jb20vRGlnaUNlcnRUcnVzdGVkRzRSU0E0MDk2U0hB
# MjU2VGltZVN0YW1waW5nQ0EuY3J0MA0GCSqGSIb3DQEBCwUAA4ICAQBVqioa80bz
# eFc3MPx140/WhSPx/PmVOZsl5vdyipjDd9Rk/BX7NsJJUSx4iGNVCUY5APxp1Mqb
# KfujP8DJAJsTHbCYidx48s18hc1Tna9i4mFmoxQqRYdKmEIrUPwbtZ4IMAn65C3X
# CYl5+QnmiM59G7hqopvBU2AJ6KO4ndetHxy47JhB8PYOgPvk/9+dEKfrALpfSo8a
# OlK06r8JSRU1NlmaD1TSsht/fl4JrXZUinRtytIFZyt26/+YsiaVOBmIRBTlClmi
# a+ciPkQh0j8cwJvtfEiy2JIMkU88ZpSvXQJT657inuTTH4YBZJwAwuladHUNPeF5
# iL8cAZfJGSOA1zZaX5YWsWMMxkZAO85dNdRZPkOaGK7DycvD+5sTX2q1x+DzBcNZ
# 3ydiK95ByVO5/zQQZ/YmMph7/lxClIGUgp2sCovGSxVK05iQRWAzgOAj3vgDpPZF
# R+XOuANCR+hBNnF3rf2i6Jd0Ti7aHh2MWsgemtXC8MYiqE+bvdgcmlHEL5r2X6cn
# l7qWLoVXwGDneFZ/au/ClZpLEQLIgpzJGgV8unG1TnqZbPTontRamMifv427GFxD
# 9dAq6OJi7ngE273R+1sKqHB+8JeEeOMIA11HLGOoJTiXAdI/Otrl5fbmm9x+LMz/
# F0xNAKLY1gEOuIvu5uByVYksJxlh9ncBjDCCBuowggTSoAMCAQICEASm0Aym578T
# 1gYB36tchgkwDQYJKoZIhvcNAQELBQAwaTELMAkGA1UEBhMCVVMxFzAVBgNVBAoT
# DkRpZ2lDZXJ0LCBJbmMuMUEwPwYDVQQDEzhEaWdpQ2VydCBUcnVzdGVkIEc0IENv
# ZGUgU2lnbmluZyBSU0E0MDk2IFNIQTM4NCAyMDIxIENBMTAeFw0yMTA2MDgwMDAw
# MDBaFw0yNDA4MzAyMzU5NTlaMG8xCzAJBgNVBAYTAkdCMRAwDgYDVQQHEwdORVdC
# VVJZMSYwJAYDVQQKEx1NaWNybyBGb2N1cyBJbnRlcm5hdGlvbmFsIHBsYzEmMCQG
# A1UEAxMdTWljcm8gRm9jdXMgSW50ZXJuYXRpb25hbCBwbGMwggGiMA0GCSqGSIb3
# DQEBAQUAA4IBjwAwggGKAoIBgQDqIkGIRgsMnPnxpiGnVbpT3NhmfZQmEMQvStb0
# kN6arPt4bsjK4yqdrV0VOB5sKWNsN7Xb2K+2aW498Gyj0euyAadCBb89CjQGLewH
# myR23Tzqmb0tNV1eaHzaufQASr0DgHLgHZlGnWC3VUWHH3TA1+A9otFkroXiEjZG
# Vt1q6QQ2Hka8idh8IaEkF24RxrX4oRhEB7c/ii6Y0sSf57Y+tHqXdYumuENkaA44
# La79BHuMavwE2fWurpE7lgflf1nnbyJxaQ+XrUgSqi4Qvd6Zwj6Dvo5pUJAuqqhw
# KYwH0n7BApxFKlZq/w6UdS1NTMohmxyqINyxA+4dKQilKUHXiXP0d+GUSrRFtXtN
# U3TKgpO2LFGZSls/l25emc2InT2mHT7SkGklwD87iM28rPS6CneVNlQcQspSy72Y
# CREJWIV1Jbv7xAXpOl8wbgVB+A2Tm5lSG5FUu79rE+n14pOom4VODzxA3atMhm40
# h383tS6V3PHyK4qLUbpLkkSDL/ECAwEAAaOCAgYwggICMB8GA1UdIwQYMBaAFGg3
# 4Ou2O/hfEYb7/mF7CIhl9E5CMB0GA1UdDgQWBBSG6FvrVWg7DnMIRIHoce297iaa
# MTAOBgNVHQ8BAf8EBAMCB4AwEwYDVR0lBAwwCgYIKwYBBQUHAwMwgbUGA1UdHwSB
# rTCBqjBToFGgT4ZNaHR0cDovL2NybDMuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0VHJ1
# c3RlZEc0Q29kZVNpZ25pbmdSU0E0MDk2U0hBMzg0MjAyMUNBMS5jcmwwU6BRoE+G
# TWh0dHA6Ly9jcmw0LmRpZ2ljZXJ0LmNvbS9EaWdpQ2VydFRydXN0ZWRHNENvZGVT
# aWduaW5nUlNBNDA5NlNIQTM4NDIwMjFDQTEuY3JsMD4GA1UdIAQ3MDUwMwYGZ4EM
# AQQBMCkwJwYIKwYBBQUHAgEWG2h0dHA6Ly93d3cuZGlnaWNlcnQuY29tL0NQUzCB
# lAYIKwYBBQUHAQEEgYcwgYQwJAYIKwYBBQUHMAGGGGh0dHA6Ly9vY3NwLmRpZ2lj
# ZXJ0LmNvbTBcBggrBgEFBQcwAoZQaHR0cDovL2NhY2VydHMuZGlnaWNlcnQuY29t
# L0RpZ2lDZXJ0VHJ1c3RlZEc0Q29kZVNpZ25pbmdSU0E0MDk2U0hBMzg0MjAyMUNB
# MS5jcnQwDAYDVR0TAQH/BAIwADANBgkqhkiG9w0BAQsFAAOCAgEAXcr+1Rhbxddq
# jgEN21fGnuoGpUwg9pscJcxH3PunJof+Yf4fTfH+kxysvey7KwzfsFgOAGlXy+KN
# D5P5WSOBV2BQuZY6sKBSANgqnKlYtn6Z1kpQ3DUrvliyK3A/UrWO+cZBD74J1bDG
# y8iIUL5TMDKUIfzJzEREONCP9512Q1ZLI6EWt36xEGEC6/MBJh6EvOpf7SeEdD/3
# Jiut5xjG/LnNS4ERKKcZQo3/zWjSvM9gYw/IiexNAdaRsq+b4kTNP/iPjowK7m7J
# llHGD5Im1+6UNXNbIQ36buH6N3qks9pyDDxJyjnia8Ur4ubV32PJiHzuLqONyg4P
# vUmYFky3uE9j9f4xEYo3N0qYhfUDmKQ45LUDwZWuAnBem+RwIv337N+U+8AjappQ
# PYQ62/unY6Ah5CmgkG21pJT7ozf0sUdo6X5KxF9ELsGe+nWfUWIfWjRXki8OK6h+
# i39gXdiY2E6cNcyjfhQeseOHWalplcxUq8cZ88cUtabbsMMsy+nUaQztNN8tDm0F
# drAqFG943hN+4n6jjn6nkq7CKrnhQsJR7Qpy4yjOik0fIspGL6F6KfFHgAxCWGMo
# vLard0mwigReH0xIy6vNglG6Y45RKlaDfKV8qBWcrfUKFxOW7MoBEUcj59OzIHzI
# S92Y+v52BvBc8iXvWMoxw+7JqBocJfQxggXTMIIFzwIBATB9MGkxCzAJBgNVBAYT
# AlVTMRcwFQYDVQQKEw5EaWdpQ2VydCwgSW5jLjFBMD8GA1UEAxM4RGlnaUNlcnQg
# VHJ1c3RlZCBHNCBDb2RlIFNpZ25pbmcgUlNBNDA5NiBTSEEzODQgMjAyMSBDQTEC
# EASm0Aym578T1gYB36tchgkwDQYJYIZIAWUDBAIBBQCggYQwGAYKKwYBBAGCNwIB
# DDEKMAigAoAAoQKAADAZBgkqhkiG9w0BCQMxDAYKKwYBBAGCNwIBBDAcBgorBgEE
# AYI3AgELMQ4wDAYKKwYBBAGCNwIBFTAvBgkqhkiG9w0BCQQxIgQguHLgtSPim4Fk
# A9BXSolx7DrHD++Tk0jehAlzYxKdpGAwDQYJKoZIhvcNAQEBBQAEggGANP9shCun
# VNEU5AcJAEqTi/bBiZh33prhyUMIEx4CkPMXBh8WPzu+Z0pjykD2keyvlYPOXeA7
# S4CCtkBjFyynDR9SL8jsQGZnPQIzHlS2+PkMrCBU0ASbZwhMU4wYPO5wOTfXoFzP
# BP5rgXZwvEp9sCKD40TbfNIkOF0pkFZGqqP+EXME6JwEu4kS3keN9GPk4qR7nCbw
# ULzs7a9m2xXmcnX9c8FWAOlSmTHntemcLqbpApVjCzUehKojRnNAPPGt3od7ymLE
# uEiEoOOTGgtklhu6AftVtpityyD85PN26d1wouIxhR6a1NI0W+2ctCul4+FX/BpL
# TUsN3VaQ9ghz7utXi1NAKkCaPNcdEEhMWoHyIEAuSDaOVsU/yB22JwqsJrXSSUzS
# 4ZtjPX3rK/0ADSOhJoxETlmj3rCJZIKOaz+f3O2Y3kYuyY2vfkb0neoGUvyhqdfY
# zPk7mus/uicnZh8sKs1WjpBETvZ3Ith3jnU4HsciHBf4NREuQPbmPIQJoYIDIDCC
# AxwGCSqGSIb3DQEJBjGCAw0wggMJAgEBMHcwYzELMAkGA1UEBhMCVVMxFzAVBgNV
# BAoTDkRpZ2lDZXJ0LCBJbmMuMTswOQYDVQQDEzJEaWdpQ2VydCBUcnVzdGVkIEc0
# IFJTQTQwOTYgU0hBMjU2IFRpbWVTdGFtcGluZyBDQQIQDE1pckuU+jwqSj0pB4A9
# WjANBglghkgBZQMEAgEFAKBpMBgGCSqGSIb3DQEJAzELBgkqhkiG9w0BBwEwHAYJ
# KoZIhvcNAQkFMQ8XDTIyMTEzMDE0NDczN1owLwYJKoZIhvcNAQkEMSIEIGu78TRY
# 3AuUBVxpQeZqUvF+fViP0lUbjx+lRKuGRLIrMA0GCSqGSIb3DQEBAQUABIICAIZ0
# GU+Krk+0UmVJVKf4IRK0KlGDBWHCDpMNS0ttF6/owdBeXRdfrCM59oN9rcj1yphH
# s3oNf8w4WnzBtUrIxOeViUyFciE/Fnl4hPOPwFRUyry+hZ3yJPBzmFhSJ99yYEYf
# vuhZrzOK7jf8th92DDxchIOI3V8vL2RLEc1KSwvmgdDF/fVKXNG306kNvWOa32AB
# yVD9buJMLYhcXhhNnkkxd67BPP5vzj4m13r2lac9QFW3y1cIDOci8wVYSIRnT9Uk
# 0zOjNWamlmQgQspxa+gf2hjjg1YQfbjS72M7CoDE4OgawrZC76sj/N7QOLFV9uhT
# D5lJJ/siWMJEXsPaZDvHxSp9P11eBN0tjTr02qayUnhza775sewzR/S0zJ9sKcr9
# CCjz895/NvPRVmj2YdYbTj4b+AOAor075WB+2wEPJLpghxE/bBx8GfItsUYfrJeM
# Rtr1z/UCfM8U6J4Bd5vViQweq/pGBjDkbh25gtZnjEbQpKgsisPoulc832Q6w5yl
# 9NBnhWwMfLdCNRfxFVXcUZPenJF/e4fWAjMOPHbH/zuceS0MLVp2UYdZL7SbbQcz
# jWGEomJd27Lzp48vYrBMu1jwBscisQqkNCX09wRJoJjRVuZNE3DxJcaKzobb6Bzk
# 46c/y2SHrEH89m/nzvjWVDhOD2b4zBn9jNDivGSh
# SIG # End signature block
